{*****************************************************************************
 *
 *  UFolderTree.pas - Folder Tree Component
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit UFolderTree;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   ComCtrls;

type

TLocationType = (LOCAL, TOBROWSER, TOFRAME);
TKind = (FolderType, LinkType);
TLinkNode = record
  Description : String;
  Notes       : TStrings;
  Kind        : TKind;
  Location    : String;
  LocationType : TLocationType;
end;

TLinkNodePtr =  ^TLinkNode;

TFolderTree = class(TTreeView)
  constructor Create(AOwner : TComponent); override;
  private
    NodDrag,LastModified                   : TTreeNode;
    Lastdata                               : TLinkNode;

    procedure TreeViewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure TreeViewDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure TreeViewDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ClearNode(Node : TTreeNode);

  protected
  public
    function AddBookMark(Location, Description : String; Notes : TStrings;
                         LocationType : TLocationType; Where : TTreeNode) : TTreeNode;
    function AddFolder(Description, Location : String; Notes : TStrings; LocationType : TLocationType; Where : TTreeNode) : TTreeNode;
    procedure ModifyItem(WhichOne : TTreeNode; Location, Description : String;
                         Notes : TStrings; cbIndex : integer);
    procedure WriteJavaScript(Filename : String);
    function ReadJavaScript(Filename : String) : boolean;

    function NodeDelete(Node : TTreeNode) : boolean;
    procedure ClearTree;
    procedure UndoLast;
    procedure DoPaste;
  end;

procedure Register;

implementation

constructor TFolderTree.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  OnMouseDown := TreeViewMouseDown;
  OnDragDrop := TreeViewDragDrop;
  OnDragOver := TreeViewDragOver;
  lastmodified := nil;
end;


procedure TFolderTree.TreeViewDragDrop(Sender, Source: TObject; X, Y: Integer);
var NodX : TTreeNode;
    HT : THitTests;
begin
  if Source is TFolderTree then
  begin
    HT := GetHitTestInfoAt(X, Y);
    NodX := TFolderTree(source).GetNodeAt(X,Y);
    if (nodX <> nil) and (HT - [htOnIcon,htOnItem, htOnLabel] <> HT) then
      if TLinkNodePtr(nodX.data)^.Kind = FolderType then
        nodDrag.MoveTo(nodX, naAddChild)
      else
        nodDrag.MoveTo(nodX,naInsert);
  end;
end;

procedure TFolderTree.TreeViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source is TFolderTree;
end;

procedure TFolderTree.TreeViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
     nodDrag := GetNodeAt(X,Y);
     if (nodDrag <> nil) and (TLinkNodePtr(nodDrag.data)^.kind = LinkType) then
       BeginDrag(false);
  end;
end;

procedure TFolderTree.DoPaste;
begin
  if lastdata.kind = LinkType then
    AddBookMark(lastdata.location, lastdata.description,lastdata.notes,
                                   lastdata.locationType, Selected);
end;

function TFolderTree.AddBookMark(Location, Description : String;
        Notes : TStrings; LocationType : TLocationType; Where : TTreeNode) : TTreeNode;
var  Addednode            : TTreeNode;
     Plink                 : TLinkNodePtr;
begin
  if Where = nil
    then Where := Selected;
  Plink := New(TLinkNodePtr);
  if Description = '' then
    Description     := Location;
  Plink^.Notes        := Notes;
  Plink^.Location     := Location;
  Plink^.Description  := Description;
  Plink^.kind         := LinkType;
  Plink^.LocationType := LocationType;
  if (Where = nil) or (TLinkNodePtr(Where.Data)^.Kind = FolderType) then
    Addednode  := Items.AddChildObject(Where, Description, Plink)
  else
    Addednode  := Items.AddChildObject(Where.Parent, Description, plink);
  Addednode.ImageIndex    := 0;
  Addednode.ImageIndex    := 2;
  Addednode.SelectedIndex := 3;
  Selected := Addednode;
  Result := Addednode;
end; { TFolderTree.AddBookMark }

function TFolderTree.AddFolder(Description, Location : String;
        Notes : TStrings; LocationType : TLocationType;
        Where : TTreeNode) : TTreeNode;
var  Addednode             : TTreeNode;
     Plink                 : TLinkNodePtr;
begin
  if Where = nil
    then where := selected;
  Plink := New(TLinkNodePtr);
  Plink^.Notes       := Notes;
  Plink^.Description :=  Description;
  Plink^.kind        := FolderType;
  Plink^.Location    := Location;
  Plink^.LocationType := LocationType;
  if (Where = nil) or (TLinkNodePtr(Where.data)^.kind = FolderType) then
    Addednode  := Items.AddChildObject(Where, Description, Plink)
  else
    Addednode  := Items.AddChildObject(Where.Parent, Description, Plink);
  Addednode.ImageIndex    := 0;
  Addednode.SelectedIndex := 1;
  Selected := Addednode;
  Result := Addednode;
end; { TFolderTree.AddFolder }

procedure TFolderTree.ModifyItem(WhichOne : TTreeNode;
                   Location, Description : String; Notes : TStrings; CBIndex : integer);
var Link : TLinkNodePtr;
begin
  if WhichOne <> nil then begin
    Link := TLinkNodePtr(WhichOne.Data);
    Lastmodified   :=  Whichone;
    Lastdata       :=  Link^;
    Link^.Description := Description;
    Link^.Notes := Notes;
    Link^.location :=  Location;
    Link^.locationType := TLocationType(CBIndex);
    WhichOne.Text  :=  Description;
  end;
end; { TFolderTree.ModifyItem }

procedure TFolderTree.UndoLast;
begin
  TLinkNodePtr(Lastmodified.Data)^ := Lastdata;
  Lastmodified.Text := Lastdata.Description;
  Invalidate;
end; { TFolderTree.UndoLast }

function TFolderTree.ReadJavaScript(Filename : String) : boolean;
  { Really a BAD parser - could use some Lex/Yacc-like improvement }
var HandleFile : TextFile;
    InString : String;
    nodes : array[0..10] of TTreeNode;
    Description, Location : String;

  function GetDescription(Localstring : String; var Ends : integer) : String;
    { Gets a quoted string inside LocalString and set the Ends index and the end of it }
  var Start : integer;
  begin
    Start := Pos('"',Localstring);
    Ends := Pos('"',Copy(Localstring,Start+1,Length(Localstring)-Start-1));
    Result := Copy(Localstring,Start+1,Ends-1);
    Ends := Ends + Start;
  end; { GetDescription }


  procedure ParseLine(localString : String);


    function GetIdentifier(Localstring : String; var Ends : integer) : Integer;
    var Foldpos, Auxpos : integer;
    begin
      Auxpos := Pos('aux',localstring); // sigh!! only recognizes aux-like identifiers
      Foldpos := Pos('FoldersTree',localstring);
        if (auxpos <= 0) then begin
            Ends := Foldpos;
            result := 0
         end
         else begin
            Ends := Auxpos;
            Result := StrToInt(Copy(LocalString,Auxpos+3,1));
         end;
    end; { GetIdentifier }

    function GetLocationType(Localstring : String) : TLocationType;
    var index : Integer;
    begin
      index := Pos('gLnk(',LocalString);
      if Index > 0 then
        Result := TLocationType(StrToInt(Localstring[index+5]))
      else
        Result := TOFRAME;
    end; { GetLocationType }

  var Current, Parent : integer;
    Ends : integer;
    Locationtype : TLocationType;

  begin
    current := GetIdentifier(LocalString,Ends);
    if (Pos('insFld',localString) > 0) then // it's even case-sensitive!
      Parent := GetIdentifier(Copy(LocalString,Ends+1,Length(Localstring)-Ends-1),Ends);
    Description := GetDescription(Localstring, Ends);
    Location := GetDescription(Copy(Localstring,Ends+1,Length(Localstring)-Ends-1), Ends);
    Locationtype := GetLocationType(LocalString);
    if Pos('insDoc',localString) > 0 then
      AddBookMark(location,description,nil, locationtype, nodes[current]);
    if (Pos('gFld',localString) > 0) then
      nodes[current] := AddFolder(description,location,nil, locationtype, nodes[parent]);
  end; { ParseLine }
var
  ends, i : integer;
  OnChangeTemp             : TTVChangedEvent;
begin
  OnChangeTemp := OnChange;
  OnChange := nil;
  ClearTree;
  for i := 0 to 10 do
    nodes[i] := nil;
  try
    AssignFile(HandleFile,FileName);
    Reset(HandleFile);
    Readln(HandleFile,InString);
    if Pos('foldersTree',InString) = 0 then
      raise EInOutError.Create('Invalid file format'); // It only keeps getting worse
    Description := GetDescription(InString, Ends);
    Location := GetDescription(Copy(InString,Ends+1,Length(InString)-Ends-1),
        Ends);
    Nodes[0] := AddFolder(Description,Location,nil,TOFRAME,nil);
    while not Eof(HandleFile) do begin
      Readln(HandleFile,Instring);
      Parseline(InString);
    end;
      FullExpand;
      Close(HandleFile);
      Result := true;
  except on EInOutError do
       Result := false;
  end;
  OnChange := OnChangeTemp;
end;  { TFolderTree.ReadJavaScript }


procedure TFolderTree.ClearNode(Node : TTreeNode);
  { Garbage collection }
var i         : integer;
    Ptr, PTro        : TLinkNodePtr;

begin
  if (node <> nil) then begin
    Ptro := TLinkNodePtr(Node.Data);
    if (Ptro <> nil) and (Ptro^.kind = FolderType) then
      for i := 0 to Node.Count-1 do begin
        ClearNode(node.item[i]);
    end;
    if (Ptro <> nil) then Dispose(Ptro);

  end;
end; { ClearNode }

procedure TFolderTree.ClearTree; { disposes nodes' data }
begin
  ClearNode(Items.GetFirstNode);
  Items.Clear;
end;

function TFolderTree.NodeDelete(Node : TTreeNode) : boolean;
var
  ptr                  : TLinkNodePtr;
begin
  if (Node = nil) then
    Node       := Selected;

  Ptr        := TLinkNodePtr(Node.Data);
  Lastdata   := ptr^;
  if (node <> nil) and (node <> Items.GetFirstNode) and ((Lastdata.Kind = LinkType)
    or (MessageDlg('Are you sure you want to delete this folder? ',
                mtConfirmation, [mbYes, mbNo], 0) = mrYes)) then begin
      ClearNode(node);
      Items.Delete(node);
      Result := true;
    end else
      Result := false;
end; { TFolderTree.NodeDelete }

procedure TFolderTree.WriteJavaScript(Filename : String);
  { does the job in a dirty way }
var HandleFile : TextFile;

  procedure WriteTree(Tn : TTreeNode);
  var Data : TLinkNode;
     OutString : String;
     i : smallint;
     VarString, VarParString : String;
     poshttp                 : integer;

    function Spaces(n : integer) : String;
    var i : integer;
    begin
      SetLength(result,n);
      for i := 1 to n do
        result[i]:= ' ';
    end; { Spaces }

  begin
    Data := TLinkNodePtr(Tn.Data)^;
    VarParString :=  'aux'+IntToStr(tn.level-1);
    VarString :='aux'+IntToStr(tn.level);
    if Tn.level = 1 then
      VarParString := 'foldersTree';
    Poshttp := Pos('http://',Data.Location);
    if Data.Kind = LinkType then begin
      if PosHttp > 0 then
        Data.Location := Copy(data.location, poshttp+7, length(data.location)-poshttp-6);
      OutString :=Spaces(tn.level*3)+'insDoc('+VarParString+',gLnk('+IntToStr(Ord(data.locationtype))+',"'+data.description+'","'+data.location+'"))';
      Writeln(HandleFile,OutString);
    end
    else begin
      if Tn.level = 0 then
        OutString := 'foldersTree = gFld("'+data.description+'","'+data.location+'")'
      else
        OutString :=Spaces(tn.level*3)+VarString+' = insFld('+VarParString+', gFld("'+data.description+'","'+data.location+'"))';
        Writeln(HandleFile,OutString);
        for i := 0 to tn.Count-1 do
          WriteTree(tn.item[i]);
    end
  end;

begin
  AssignFile(HandleFile,FileName);
  Rewrite(HandleFile);
  WriteTree(Items.GetFirstNode);
  Flush(HandleFile);
  Close(HandleFile);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TFolderTree]);
end;

end.

