{*****************************************************************************
 *
 *  ChildWin.pas - Main Form
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit Childwin;

interface

uses Windows, Classes, Graphics, Forms, Controls, StdCtrls, ExtCtrls,
  ComCtrls, Menus, UFolderTree, sysutils, DropSource, DropTarget, dialogs,
  DropURLTarget, shellapi, Buttons, Global, UOptionsDialog, inifiles;

type
  TMainForm = class(TForm)
    LocationEdit: TEdit;
    NotesMemo: TMemo;
    LocationLabel: TLabel;
    NotesLabel: TLabel;
    FolderImageList: TImageList;
    DescriptionLabel: TLabel;
    DescriptionEdit: TEdit;
    PopupMenu: TPopupMenu;
    NewBookmark: TMenuItem;
    NewFolder: TMenuItem;
    TreeView: TFolderTree;
    DropURLTarget: TDropURLTarget;
    DeleteMenuItem: TMenuItem;
    ComboBox: TComboBox;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    MainMenu: TMainMenu;
    FileMenuItem: TMenuItem;
    LoadMenuItem: TMenuItem;
    SaveMenuItem: TMenuItem;
    PreviewMenuItem: TMenuItem;
    EditMenuItem: TMenuItem;
    AddLinkMenuItem: TMenuItem;
    AddFolderMenuItem: TMenuItem;
    ChangeMenuItem: TMenuItem;
    UndoChangeMenuItem: TMenuItem;
    ClearMenuItem: TMenuItem;
    ExitMenuItem: TMenuItem;
    HelpMenuItem: TMenuItem;
    ContentsMenuItem: TMenuItem;
    AboutMenuItem: TMenuItem;
    NewMenuItem: TMenuItem;
    Panel: TPanel;
    NewSpeedButton: TSpeedButton;
    OpenSpeedButton: TSpeedButton;
    SaveSpeedButton: TSpeedButton;
    PreviewSpeedButton: TSpeedButton;
    ExitSpeedButton: TSpeedButton;
    LinkSpeedButton: TSpeedButton;
    FolderSpeedButton: TSpeedButton;
    ChangeSpeedButton: TSpeedButton;
    CutSpeedButton: TSpeedButton;
    N1: TMenuItem;
    CutMenuItem: TMenuItem;
    PasteMenuItem: TMenuItem;
    PasteSpeedButton: TSpeedButton;
    UndoSpeedButton: TSpeedButton;
    N2: TMenuItem;
    DeploymentMenuItem: TMenuItem;
    DeploymentSpeedButton: TSpeedButton;
    LinkTypeStaticText: TStaticText;
    ContentsSpeedButton: TSpeedButton;
    AboutSpeedButton: TSpeedButton;
    PastePopupMenuItem: TMenuItem;
    IniOpenDialog: TOpenDialog;
    IniSaveDialog: TSaveDialog;
    LeftPanel: TPanel;
    RightPanel: TPanel;
    Splitter: TSplitter;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure AddLinkButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure ChangeButtonClick(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);


    procedure NewBookmarkClick(Sender: TObject);
    procedure NewFolderClick(Sender: TObject);
    procedure DropURLTargetDrop(Sender: TObject; DragType: TDragType;
      Point: TPoint);
    procedure SaveButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
    procedure DropURLTargetDragOver(Sender: TObject; pt: TPoint);
    procedure PreviewButtonClick(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure AddFolderButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure UndoButtonClick(Sender: TObject);
    procedure NewMenuItemClick(Sender: TObject);
    procedure ExitMenuItemClick(Sender: TObject);
    procedure PasteMenuItemClick(Sender: TObject);
    procedure AboutMenuItemClick(Sender: TObject);
    procedure DeploymentMenuItemClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ContentsMenuItemClick(Sender: TObject);
    procedure ReadProject;
    procedure WriteProject;
  private
    FormerHeight, FormerWidth : integer;
    Foldernode                : TTreeNode;
    EnabledPaste              : boolean;
    Firstcall                 : boolean;
    OldWindowState            : TWindowState;
    IlFIleIni                 : TIniFile;

  public
    { Public declarations }
  end;
var
    MainForm : TMainForm;
implementation

uses about;

function GetWinDir : String;
var WindowString  : String;
begin
  SetLength(WindowString,250+1);
  SetLength(WindowString,GetWindowsDirectory(PChar(WindowString),MAX_PATH));
  result := WindowString;
end;

procedure TMainForm.ReadProject;
begin
  Directory  := IlFIleIni.ReadString('Main','directory',GetCurrentDir+'\Deployment');
  MainHtml   := IlFileIni.ReadString('Main','MainHtml','index.html');
  JavaScript := IlFIleIni.ReadString('Main','JavaScript','tree.js');
  LeftFrame  := IlFIleIni.ReadString('Main','LeftFrame','leftframe.html');
  RightFrame  := IlFIleIni.ReadString('Main','RightFrame','home.html');
  if not TreeView.ReadJavaScript(directory+'\'+JavaScript) then
    Foldernode :=  TreeView.AddFolder('Folder Tree','Folder Tree', nil, TOFRAME, nil);
end;

procedure TMainForm.WriteProject;
begin
  IlFileIni.WriteString('Main','directory',directory);
  IlFIleIni.WriteString('Main','MainHtml',MainHtml);
  IlFIleIni.WriteString('Main','JavaScript',Javascript);
  IlFIleIni.WriteString('Main','LeftFrame',leftframe);
  IlFileIni.WriteString('Main','RightFrame',rightframe);
end;

{$R *.DFM}

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
  procedure WriteIni;
  begin
    WriteProject;
  end;

begin
  DropURLTarget.UnRegister;
  Action := caFree;
 // TreeView.ClearTree;
  WriteIni;
end;

procedure TMainForm.FormCreate(Sender: TObject);

  procedure ReadIni;
  var IniFile :String;
  begin
     IniFile := GetWinDir+'\ftcreator.ini';
     StartDir := GetCurrentDir;
     ILFileIni := TIniFile.Create(IniFile);
     ReadProject;
  end;

begin
  OldWindowState := Windowstate;
  DropURLTarget.Register(LocationEdit);
  DropURLTarget.Register(TreeView);
  Firstcall          := true;
  ScaleControls(Screen.Height, 480);
  EnabledPaste      := False;
  ReadIni;
  FormerHeight       := Height;
  FormerWidth        := Width;
end;


procedure TMainForm.AddLinkButtonClick(Sender: TObject);
begin
  TreeView.AddBookMark(LocationEdit.Text, DescriptionEdit.Text, NotesMemo.Lines, TLocationType(ComboBox.ItemIndex),nil)
end;

procedure TMainForm.DeleteButtonClick(Sender: TObject);
begin
  if TreeView.Selected <> foldernode then
    EnabledPaste := TreeView.NodeDelete(TreeView.Selected);
end;

procedure TMainForm.TreeViewChange(Sender: TObject; Node: TTreeNode);
var Link : TLinkNode;
begin
  try
    Link := TLinkNodePtr(Node.Data)^;
    with Link do begin
      DescriptionEdit.Text := description;
      if Notes <> nil then
        NotesMemo.Lines.AddStrings(Notes);
      LocationEdit.Text := location;
      ComboBox.ItemIndex := Ord(locationtype);
    end;
  except on EAccessViolation do;
  end;
end;

procedure TMainForm.ChangeButtonClick(Sender: TObject);
begin
  TreeView.ModifyItem(TreeView.Selected,LocationEdit.Text, DescriptionEdit.Text, NotesMemo.Lines, ComboBox.ItemIndex);
end;

procedure TMainForm.NewBookmarkClick(Sender: TObject);
begin
  TreeView.AddBookMark('New Bookmark','New Bookmark',nil, TOFRAME, nil);
end;

procedure TMainForm.NewFolderClick(Sender: TObject);
begin
  TreeView.AddFolder('New Folder','New Folder',nil, TOFRAME, nil);
end;

procedure TMainForm.DropURLTargetDrop(Sender: TObject;
  DragType: TDragType; Point: TPoint);
begin
  if (DropURLTarget.URL) <> '' then
     if (Sender is TEdit) then begin
         LocationEdit.Text := DropURLTarget.URL;
         if Length(LocationEdit.Text) > 0 then
           LocationEdit.Invalidate;
     end
     else
        TreeView.AddBookmark(DropURLTarget.URL,DropURLTarget.URL, nil, TOFRAME,
          TreeView.GetNodeAt(ScreenToClient(Point).X-TreeView.Left,ScreenToClient(Point).Y-TreeView.Top));
end;

procedure TMainForm.SaveButtonClick(Sender: TObject);
begin
  if (SaveDialog.Execute) then
    if Pos('.js',SaveDialog.FileName) > 0 then
      TreeView.WriteJavaScript(SaveDialog.FileName)
    else
      TreeView.WriteJavaScript(SaveDialog.FileName+'.js');
end;

procedure TMainForm.LoadButtonClick(Sender: TObject);
begin
  if (OpenDialog.Execute) then
    if not TreeView.ReadJavaScript(OpenDialog.FileName) then
      Foldernode        :=  TreeView.AddFolder('Folder Tree','Folder Tree', nil, TOFRAME, nil);
    FolderNode := TreeView.Items.GetFirstNode;
end;


procedure TMainForm.DropURLTargetDragOver(Sender: TObject; pt: TPoint);
begin
  TreeView.GetNodeAt(ScreenToClient(pt).x-TreeView.Left,ScreenToClient(pt).y-TreeView.Top).Selected := True;
end;

procedure TMainForm.PreviewButtonClick(Sender: TObject);
var Html : String;

  procedure WriteLF;
  var HandleFile : TextFile;
  begin
    AssignFile(HandleFile,Directory+'\'+LeftFrame);
    Rewrite(HandleFile);
    Writeln(HandleFile,'<html><head></head>');
    Writeln(HandleFile,'<script src="ftiens4.js"></script>');
    Writeln(HandleFile,'<script src="'+Javascript+'"></script><script>');
    Writeln(HandleFile,'initializeDocument()');
    Writeln(HandleFile,'</script></body></html>');
    CloseFile(HandleFile);
  end; { WriteLF }

  procedure WriteIndex;
  var HandleFile : TextFile;
  begin
    AssignFile(HandleFile,Directory+'\'+MainHtml);
    Rewrite(HandleFile);
    Writeln(HandleFile,'<html><head></head>');
    Writeln(HandleFile,'<FRAMESET cols="200,*">');
    Writeln(HandleFile,'<FRAME src="'+LeftFrame+'" name="treeframe" >');
    Writeln(HandleFile,'<FRAME SRC="'+RightFrame+'" name="basefrm">');
    Writeln(HandleFile,'</FRAMESET></HTML>');
    CloseFile(HandleFile);
  end; { WriteIndex }

begin
  TreeView.WriteJavaScript(Directory+'\'+Javascript);
  Html := Directory+'\'+MainHtml;
  WriteLF;
  WriteIndex;
  Shellexecute(0,nil,Pchar(Html),nil,nil,SW_NORMAL);
end;

procedure TMainForm.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Ord(Key) = 13 then
    ChangeButtonClick(self);
end;

procedure TMainForm.AddFolderButtonClick(Sender: TObject);
begin
  TreeView.AddFolder(DescriptionEdit.Text, LocationEdit.Text, NotesMemo.Lines, TLocationType(ComboBox.ItemIndex),nil)
end;

procedure TMainForm.ClearButtonClick(Sender: TObject);
begin
  LocationEdit.Clear;
  DescriptionEdit.Clear;
  NotesMemo.Clear;
end;

procedure TMainForm.UndoButtonClick(Sender: TObject);
begin
  TreeView.UndoLast;
end;

procedure TMainForm.NewMenuItemClick(Sender: TObject);
begin
  if MessageDlg('Do you want to save changes?', mtConfirmation,  [mbYes, mbNo], 0) = mrYes then
    SaveDialog.Execute;
  TreeView.Items.Clear;
  Foldernode := TreeView.AddFolder('Folder Tree','Folder Tree', nil, TOFRAME, nil);
end;

procedure TMainForm.ExitMenuItemClick(Sender: TObject);
begin
  if MessageDlg('Do you want to save changes?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    SaveDialog.Execute;
  Close;
end;

procedure TMainForm.PasteMenuItemClick(Sender: TObject);
begin
  if (enabledPaste) then
    TreeView.DoPaste;
  enabledPaste := False;
end;

procedure TMainForm.AboutMenuItemClick(Sender: TObject);
begin
  AboutBox.Show;
end;

procedure TMainForm.DeploymentMenuItemClick(Sender: TObject);
begin
  OptionsDlg.Show;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  if windowstate = oldwindowstate then begin
    if (not firstcall) then
      ScaleControls(round(sqrt(Width*Height)), round(sqrt(FormerWidth*FormerHeight)));
  end else
    if windowstate = wsNormal then
      ScaleControls(Height,Screen.Height)
    else
      ScaleControls(Screen.Height,FormerHeight);
  FormerHeight   := Height;
  FormerWidth    := Width;
  Oldwindowstate := windowstate;
end;

procedure TMainForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  firstcall := false;
  onResize := FormResize;
  OnMouseMove := nil;
end;

procedure TMainForm.ContentsMenuItemClick(Sender: TObject);
begin
  HelpFile := StartDir+'\help\ftexample.html';
  Shellexecute(0,nil,Pchar(HelpFile),nil,nil,SW_NORMAL);
end;

end.

